/*
    ATTiny44 mit 1MHz

	LED-Paneelmeter V2.0
	Version ohne Fliekommaarithmetik

    Letzte Modifikation: 15.05.2013
    Author: Falko Glanert, DG1AFG

    Optimization: -Os

	Kalibiermod.			=> 	PA5 (MISO)
	Betriebsartumschaltg.	=>	PA0

	Taktfrequenz:				1MHz
*/
// -------------------------------------------------------------------------------------------------
// Maximalwert (Spannungsbereich*10) * Korrekturfaktor Widerstands-Vorteiler (33k / 1k => 34:1
// 34 : 24V = 1,4
#define Uemax 240 * 1.4
// Strombereich; 10mOhm + OPV Vu=5
#define Iemax 255
// -------------------------------------------------------------------------------------------------

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

// Timer0 Reload-Wert
const uint8_t TCNT0_Reload = 225;		// Timer Reload fr 2ms

uint16_t eeUmaxRef EEMEM = Uemax;		// Maximalwert der Eingangsspannung im EEPROM (entspr. Uref mit Spannungsteiler)
uint16_t UmaxRef;						// dito im Ram
uint16_t eeImaxRef EEMEM = Iemax;		// Maximalwert Strom
uint16_t ImaxRef;						// dito im Ram
volatile uint16_t ADC_result;			// Puffer fr ADC-Wert
uint8_t bcd[5]; 						// Array fr BCD Stellen ( int2bcd() )
uint32_t Measure_Value;					// errechneter Messwert (10,0V = 100)

// Zeichenspeicher (0->links)
volatile uint8_t Digit[3];				// Zeichenspeicher fr 7-Segment

#include "includes/hardware.h"
#include "includes/adc-routines.c"
#include "includes/BitUtilities.h"

// ----------------------------------------------
// Wandelt 16 Bit Integer in einzelne BCD Stellen
// ----------------------------------------------
void int2bcd(uint16_t zahl)
{
    uint8_t i;
    for(i=0; i<5; i++)
        {
            bcd[i] = zahl % 10;				//Ablage in's Array [0]=LSB
            zahl /= 10;
        }
}
// -----------------------
// Hardware initialisieren
// -----------------------
void init_hardware (void)
{
    //Ports initialisieren
    DDRA= 0xdc; //Bit0,1,5 = Eingnge fr ADC(Bit 5 = MISO => Calibrate!)
    DDRB= 0xff;
    PORTA=0xfd;
    PORTB=0xff;
    ADC_Init();											// ADC initialisieren
    // Timer0 settings:
    TCCR0B = (1<<CS01) | (1<<CS00); 					// CLK/64
    TIMSK0 = (1 << TOIE0); 								// Timer/Counter0 Overflow Interrupt Enable
    TCNT0 = TCNT0_Reload;
}
// ------------------
// --- ISR Timer0 ---
// --- alle 2ms ---
// ------------------
ISR(TIM0_OVF_vect) 										// Timer/Counter0 Overflow
{
#include "includes/LTC4622_Handler.h"
}

// ------------------
// --- Einsprung ----
// ------------------
int main(void)
{
    init_hardware();
	sei(); 												// enable interrupts
    if (!(PINA & 1<<Calibrate))							// nach Einschalten Kalibriermodus ?
        {
            ADC_result=ADC_Read_Avg(1,4);				// AD-Wandlung (10-bit, 4-fach gemittelt, Kanal 1)
            if (!(PINA & 1<<Mode_Pin))
                {
                    // Stromkalibrierung
                    ImaxRef = 20480/ADC_result;				// Referenz bei Ie=2A ermitteln
                    eeprom_write_word(&eeImaxRef, ImaxRef);	// und im EEPROM speichern
                }
            else
                {
                    // Spannungskalibrierung
                    UmaxRef = 102400/ADC_result;			// Referenz bei Ue=10V ermitteln
                    eeprom_write_word(&eeUmaxRef, UmaxRef);	// und im EEPROM speichern
                }
			Digit[0] = 0;								// Strich auf's Display
            Digit[1] = 10;								// => Da PA5 durch Drahtbrcke immer noch auf Masse liegt, werden Segmente
            Digit[2] = 10;								// F,C mit eingeschaltet. (Sieht aus wie SS-Ruhne)
            while(1) {}									// Endlosschleife
        }

    UmaxRef = eeprom_read_word(&eeUmaxRef);				// Referenzwert U aus EEPROM holen
    ImaxRef = eeprom_read_word(&eeImaxRef);				// Referenzwert I aus EEPROM holen
    DDRA= 0xfc; 										// Bit0,1 = Eingnge fr ADC (Calibrate Pin wegschalten & zum Ausgang machen


//Hauptschleife
// A/D Wandlung Spannung wird wegen Strunterdrckung besser in der ISR vorgenommen (Dunkelphase Display)
    while(1)
        {
            // Strom- oder Spannungsmessung?
            if (!(PINA & 1<<Mode_Pin))
                {
                    // Strommessung
                    Measure_Value = (uint32_t)ADC_result * ImaxRef;	// Strom errechnen (typecast, weil C sonst nur mit dem kleinen Zahlentyp rechnet => berlauf!)
                }
            else
                {
                    //Spannungsmessung
                    Measure_Value = (uint32_t)ADC_result * UmaxRef;	// Spannung errechnen (typecast, weil C sonst nur mit dem kleinen Zahlentyp rechnet => berlauf!)
                }

            Measure_Value /= 1024;


            // berlauf ?
            if (Measure_Value < 200)
                {
                    int2bcd((uint16_t)Measure_Value);	// BCD Konvertierung; => ins Array
                    Digit[0] = bcd[2];
                    Digit[1] = bcd[1];
                    Digit[2] = bcd[0];
                }
            else										// berlauf !
                {
                    Digit[0] = 0;						// Strich auf's Display
                    Digit[1] = 10;
                    Digit[2] = 10;
                }

            _delay_ms(250);

        }
}

